-- jflowmap
-- record selection #40 (AT-CEEPUS 2003..2013)
-- only co-publication links between capitals of the countries
-- with ISO-639-3 country codes
-- for a selected subject field


DROP TABLE IF EXISTS temp_cities;
SELECT DISTINCT
        r.id, os.cc, regexp_replace(ci.capital, ' ', '_', 'g') as city, r.year, cic.ccc
INTO TEMP temp_cities
FROM
        (SELECT DISTINCT r.id, r.id_wos, r.id_scopus, r.year
           FROM record AS r,
                study_record AS sr,
                (select array_agg(y) as years from generate_series(2003,2013) y) AS period1
           WHERE r.id=sr.record_id
             AND sr.study_id=40
             AND r.year = ANY(period1.years)
             AND sm_subfield in (select id3 from classif_node where scheme_id=1 and level=3 and id2='10')
        ) r,
        corrdata_org co,
        org_string os,
        countryinfo ci,
        country_iso_codes cic
WHERE
        COALESCE(r.id_scopus, r.id_wos)=co.identifier
    AND co.string_id=os.id
    AND os.cc IS NOT NULL
    AND os.cc=ci.cc
    AND os.cc=cic.cc
;


DROP TABLE IF EXISTS temp_jflowmap_cities;
SELECT
    ci.cc,
    regexp_replace(ci.capital, ' ', '_', 'g') AS city,
    ci.latitude AS lat,
    ci.longitude AS lon,
    cic.ccc as ccc
INTO TEMP temp_jflowmap_cities
FROM countryinfo ci, country_iso_codes cic
WHERE ci.cc=cic.cc
;


\a
\t
\f '\t'
\o /home/copub/studies/40/jflowmap/by_l2/jflowmap_40_cities_by-year_nodes_10.csv
SELECT '"ID","Name",Lat,Lon';
SELECT
    '"' || ccc || '_' || city || '","' || city || '",' || lat || ',' || lon
FROM
    temp_jflowmap_cities
;
\a
\t
\o




DROP TABLE IF EXISTS temp_link_freq;
SELECT
    r1.ccc AS ccc1,
    r1.city AS city1,
    r2.ccc AS ccc2,
    r2.city AS city2,
    r1.year,
    count(*) n
INTO TEMP temp_link_freq
FROM
    (select id, cc, city, year, ccc from temp_cities) r1,
    (select id, cc, city, year, ccc from temp_cities) r2
WHERE
        r1.id=r2.id
    AND r1.cc IN ('AT','AL','BA','BG','CZ','HR','HU','MD','ME','MK','PL','RO','RS','SI','SK','XK')
    AND (    (r2.cc IN ('AT','AL','BA','BG','CZ','HR','HU','MD','ME','MK','PL','RO','RS','SI','SK','XK') AND r2.cc > r1.cc)
          OR (r2.cc NOT IN ('AT','AL','BA','BG','CZ','HR','HU','MD','ME','MK','PL','RO','RS','SI','SK','XK'))
        )
GROUP BY
    r1.ccc, r1.city, r2.ccc, r2.city, r1.year
;



\a
\t
\f '\t'
\o /home/copub/studies/40/jflowmap/by_l2/jflowmap_40_cities_by-year_links_10.csv
SELECT '"Country1","Country2",2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013';
SELECT
    '"' || ccc1 || '","' || ccc2 || '",' ||
    array_to_string(avals(hstore(array_agg(year)::text[], array_agg(n)::text[])), ',')
FROM
   (
        SELECT
            ccc1, city1, ccc2, city2, year, n
        FROM
            temp_link_freq tlf
    UNION
        SELECT
            ccc1, city1, ccc2, city2, y.y as year, 0
        FROM
            (SELECT DISTINCT ccc1, city1, ccc2, city2, array_agg(year) years
             FROM temp_link_freq
             GROUP BY ccc1, city1, ccc2, city2
            ) t1,
            (SELECT generate_series(2003,2013) y) y
        WHERE
            not y.y=any(t1.years)
   ) tfull
GROUP BY
    ccc1, city1, ccc2, city2
ORDER BY
    ccc1, city1, ccc2, city2
;
\a
\t
\o




-- All years


\a
\t
\f '\t'
\o /home/copub/studies/40/jflowmap/by_l2/jflowmap_40_cities_nodes_10.csv
SELECT 'ID,Name,Lat,Lon';
SELECT
    '"' || ccc || '_' || city || '","' || city || '",' || lat || ',' || lon
FROM
    temp_jflowmap_cities
;
\a
\t
\o





\a
\t
\f '\t'
\o /home/copub/studies/40/jflowmap/by_l2/jflowmap_40_cities_links_10.csv
SELECT 'Country1,Country2,value';
SELECT
    '"' || ccc1 || '","' || ccc2 || '",' || n::VARCHAR
FROM
(
    SELECT
        r1.ccc AS ccc1,
        r1.city AS city1,
        r2.ccc AS ccc2,
        r2.city AS city2,
        count(*) n
    FROM
        temp_cities r1,
        temp_cities r2
    WHERE
            r1.id=r2.id
        AND r1.cc IN ('AT','AL','BA','BG','CZ','HR','HU','MD','ME','MK','PL','RO','RS','SI','SK','XK')
        AND (    (r2.cc IN ('AT','AL','BA','BG','CZ','HR','HU','MD','ME','MK','PL','RO','RS','SI','SK','XK') AND r2.cc > r1.cc)
              OR (r2.cc NOT IN ('AT','AL','BA','BG','CZ','HR','HU','MD','ME','MK','PL','RO','RS','SI','SK','XK'))
            )
    GROUP BY
        r1.ccc, r1.city, r2.ccc, r2.city
) links
;
\a
\t
\o



